@@ -22,6 +22,21 @@ def is_old_openblas(lib_controller):
     return lib_controller.internal_api == "openblas" and lib_controller.version is None
 
 
+def skip_if_openblas_openmp():
+    """Helper to skip tests with side effects when OpenBLAS has the OpenMP
+    threading layer.
+    """
+    if any(
+        lib_controller.internal_api == "openblas"
+        and lib_controller.threading_layer == "openmp"
+        for lib_controller in ThreadpoolController().lib_controllers
+    ):
+        pytest.skip(
+            "Setting a limit on OpenBLAS when using the OpenMP threading layer also "
+            "impact the OpenMP library. They can't be controlled independently."
+        )
+
+
 def effective_num_threads(nthreads, max_threads):
     if nthreads is None or nthreads > max_threads:
         return max_threads
@@ -196,6 +211,10 @@ def test_threadpool_limits_manual_restore():
 def test_threadpool_controller_limit():
     # Check that using the limit method of ThreadpoolController only impact its
     # library controllers.
+
+    # This is not True for OpenBLAS with the OpenMP threading layer.
+    skip_if_openblas_openmp()
+
     blas_controller = ThreadpoolController().select(user_api="blas")
     original_openmp_info = ThreadpoolController().select(user_api="openmp").info()
 
@@ -212,6 +231,34 @@ def test_threadpool_controller_limit():
         assert openmp_info == original_openmp_info
 
 
+def test_get_params_for_sequential_blas_under_openmp():
+    # Test for the behavior of get_params_for_sequential_blas_under_openmp.
+    controller = ThreadpoolController()
+    original_info = controller.info()
+
+    params = controller._get_params_for_sequential_blas_under_openmp()
+
+    if controller.select(
+        internal_api="openblas", threading_layer="openmp"
+    ).lib_controllers:
+        assert params["limits"] is None
+        assert params["user_api"] is None
+
+        with controller.limit(limits="sequential_blas_under_openmp"):
+            assert controller.info() == original_info
+
+    else:
+        assert params["limits"] == 1
+        assert params["user_api"] == "blas"
+
+        with controller.limit(limits="sequential_blas_under_openmp"):
+            assert all(
+                lib_info["num_threads"] == 1
+                for lib_info in controller.info()
+                if lib_info["user_api"] == "blas"
+            )
+
+
 def test_nested_limits():
     # Check that exiting the context manager properly restores the original limits even
     # when nested.
@@ -245,7 +292,7 @@ def test_threadpool_limits_bad_input():
         threadpool_limits(limits=1, user_api="wrong")
 
     with pytest.raises(
-        TypeError, match="limits must either be an int, a list or a dict"
+        TypeError, match="limits must either be an int, a list, a dict, or"
     ):
         threadpool_limits(limits=(1, 2, 3))
 
@@ -373,6 +420,8 @@ def test_nested_prange_blas(nthreads_outer):
 
     check_nested_prange_blas = prange_blas.check_nested_prange_blas
 
+    skip_if_openblas_openmp()
+
     original_info = ThreadpoolController().info()
 
     blas_controller = ThreadpoolController().select(user_api="blas")
@@ -411,7 +460,7 @@ def test_nested_prange_blas(nthreads_outer):
 
 # the method `get_original_num_threads` raises a UserWarning due to different
 # num_threads from libraries with the same `user_api`. It will be raised only
-# in the CI job with 2 openblas (py37_pip_openblas_gcc_clang). It is expected
+# in the CI job with 2 openblas (py38_pip_openblas_gcc_clang). It is expected
 # so we can safely filter it.
 @pytest.mark.filterwarnings("ignore::UserWarning")
 @pytest.mark.parametrize("limit", [1, None])
@@ -552,8 +601,11 @@ def test_architecture():
         # XXX: add more as needed by CI or developer laptops
         "armv8",
         "Haswell",
+        "Prescott",  # see: https://github.com/xianyi/OpenBLAS/pull/3485
         "SkylakeX",
         "Sandybridge",
+        "VORTEX",
+        "Zen",
     )
     expected_blis_architectures = (
         # XXX: add more as needed by CI or developer laptops
@@ -620,3 +672,33 @@ def test_threadpool_controller_as_decorator():
     outer_func()
 
     assert ThreadpoolController().info() == original_info
+
+
+def test_custom_controller():
+    # Check that a custom controller can be used to change the number of threads
+    # used by a library.
+    try:
+        import tests._pyMylib  # noqa
+    except:
+        pytest.skip("requires my_thread_lib to be compiled")
+
+    controller = ThreadpoolController()
+    original_info = controller.info()
+
+    mylib_controller = controller.select(user_api="my_threaded_lib")
+
+    # my_threaded_lib has been found and there's 1 matching shared library
+    assert len(mylib_controller.lib_controllers) == 1
+    mylib_controller = mylib_controller.lib_controllers[0]
+
+    # we linked against my_threaded_lib v2.0 and by default it uses 42 thread
+    assert mylib_controller.version == "2.0"
+    assert mylib_controller.num_threads == 42
+
+    # my_threaded_lib exposes an additional info "some_attr":
+    assert mylib_controller.info()["some_attr"] == "some_value"
+
+    with controller.limit(limits=1, user_api="my_threaded_lib"):
+        assert mylib_controller.num_threads == 1
+
+    assert ThreadpoolController().info() == original_info
